package com.fuint.common.service.impl;


import cn.hutool.core.lang.Validator;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fuint.common.dto.ext.ShopAlertInfo;
import com.fuint.common.enums.AlertEnum;
import com.fuint.common.enums.PayStatusEnum;
import com.fuint.common.enums.StatusEnum;
import com.fuint.common.service.AlertService;
import com.fuint.common.service.EmailService;
import com.fuint.common.service.StoreService;
import com.fuint.repository.mapper.MtOrderMapper;
import com.fuint.repository.model.MtOrder;
import com.fuint.repository.model.MtStore;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
public class AlertServiceImpl implements AlertService {
	private EmailService emailService;

	private StoreService storeService;

	private MtOrderMapper mtOrderMapper;


	/**
	 * 检查当日营业额并发送预警邮件
	 */
	@Override
	public void storeAlertsJob() {
		// 0. 获取时间范围（统一时间变量）
		Set<Integer> alertIds = AlertEnum.getAlertIds();
		// 1. 获取所有店铺
		List<MtStore> shops = storeService.list(new LambdaQueryWrapper<MtStore>()
				.eq(MtStore::getNextReminderDay, LocalDate.now())
				.eq(MtStore::getStatus, StatusEnum.ENABLED.getKey())
				.in(MtStore::getAlertId, alertIds));
		List<Long> shopIds = shops.stream().map(MtStore::getId).collect(Collectors.toList());

		// 2. 批量查询所有相关订单（优化性能）
		Map<Long, List<MtOrder>> ordersByShop = shopIds.isEmpty() ?
				Collections.emptyMap() :
				mtOrderMapper.selectList(
						new LambdaQueryWrapper<MtOrder>()
								.in(MtOrder::getStoreId, shopIds)
								.eq(MtOrder::getPayStatus, PayStatusEnum.SUCCESS.getKey())
				).stream().collect(Collectors.groupingBy(MtOrder::getStoreId));

		// 3. 构建预警数据（包含订单明细）
		Map<String, List<ShopAlertInfo>> alertMap = shops.stream()
				.map(shop -> {
					// 计算总销售额
					BigDecimal sales = Optional.ofNullable(ordersByShop.get(shop.getId()))
							.orElse(Collections.emptyList())
							.stream()
							.map(MtOrder::getAmount)
							.reduce(BigDecimal.ZERO, BigDecimal::add);

					// 注入订单明细
					return new ShopAlertInfo(
							shop,
							sales,
							ordersByShop.getOrDefault(shop.getId(), Collections.emptyList())
					);
				})
				.filter(info -> info.getTotalSales().compareTo(info.getShop().getThreshold()) < 0)
				.collect(Collectors.groupingBy(info -> info.getShop().getMail()));

		// 4. 发送邮件（逻辑保持不变）
		CompletableFuture.runAsync(()->{
			alertMap.forEach((mail, list) -> {
				if (Validator.isEmail(mail)) {
					emailService.sendAggregatedAlertEmail(mail, list);
				}
			});
		});

		//重新刷新店铺预警时间点
		List<MtStore> stores = shops.stream().peek(shop -> shop.setNextReminderDay(LocalDate.now()
				.plusDays(AlertEnum.getDaysByAlertId(shop.getAlertId())))).collect(Collectors.toList());
		storeService.updateBatchById(stores);
	}
}
