package com.fuint.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fuint.common.dto.ext.AlertDTO;
import com.fuint.common.dto.ext.PageResult;
import com.fuint.common.dto.ext.SelectDto;
import com.fuint.common.dto.ext.ShopAlertRequest;
import com.fuint.common.enums.AlertEnum;
import com.fuint.common.service.StoreAlertService;
import com.fuint.common.util.I18nUtil;
import com.fuint.common.vo.OrderVO;
import com.fuint.common.vo.ShopDetailVO;
import com.fuint.common.vo.ShopVO;
import com.fuint.framework.exception.BusinessCheckException;
import com.fuint.repository.mapper.MtOrderMapper;
import com.fuint.repository.mapper.MtStoreMapper;
import com.fuint.repository.model.MtOrder;
import com.fuint.repository.model.MtStore;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author Survive
 * @date 2025/3/12
 * @description TODO
 */
@Service
@AllArgsConstructor
public class StoreAlertServiceImpl extends ServiceImpl<MtStoreMapper, MtStore> implements StoreAlertService {


    private MtStoreMapper shopMapper;

    private MtOrderMapper orderMapper;

    @Override
    @Transactional
    public void addShopAlert(ShopAlertRequest alertRequest) throws BusinessCheckException {
        // 1. 校验店铺存在性
        MtStore shop = shopMapper.selectById(alertRequest.getId());
        if (shop == null) {
            throw new BusinessCheckException(I18nUtil.getMessage("storeNonexistent"));
        }
        // 2. 更新字段
        shop.setMail(alertRequest.getMail());
        shop.setThreshold(alertRequest.getThreshold());
        shop.setAlertId(alertRequest.getAlertId());
        Integer daysByAlertId = AlertEnum.getDaysByAlertId(alertRequest.getAlertId());
        if (Objects.nonNull(daysByAlertId)){
            shop.setNextReminderDay(LocalDate.now().plusDays(daysByAlertId));
        }else{
            shop.setNextReminderDay(LocalDate.now().minusDays(1));
        }
        // 3. 持久化到数据库
       shopMapper.updateById(shop);
    }

    @Override
    @Transactional
    public MtStore updateShopAlert(ShopAlertRequest alertRequest) throws BusinessCheckException{
        // 1. 验证店铺存在性
        MtStore shop = shopMapper.selectById(alertRequest.getId());
        if (shop == null) {
            throw new BusinessCheckException(I18nUtil.getMessage("storeNonexistent"));
        }

        // 2. 数据更新
        shop.setMail(alertRequest.getMail());
        shop.setThreshold(alertRequest.getThreshold());
        shop.setAlertId(alertRequest.getAlertId());
        int rows = shopMapper.updateById(shop);
        if (rows == 0){
            throw new BusinessCheckException(I18nUtil.getMessage("updateFail"));
        }
        return shop;
    }

    @Override
    @Transactional
    public MtStore clearAlertConfig(Long shopId) throws BusinessCheckException{
        // 1. 验证店铺存在性
        MtStore shop = shopMapper.selectById(shopId);
        if (shop == null) {
            throw new BusinessCheckException(I18nUtil.getMessage("storeNonexistent"));
        }

        // 2. 清除预警配置
        shop.setMail("");          // 清空邮箱
        shop.setThreshold(BigDecimal.ZERO);  // 重置阈值
        shop.setAlertId(null);
        shop.setUpdateTime(new Date());
        int rows = shopMapper.update(null,new LambdaUpdateWrapper<MtStore>()
                .set(MtStore::getAlertId,null)
                .set(MtStore::getMail,null)
                .set(MtStore::getThreshold,BigDecimal.ZERO)
                .eq(MtStore::getId,shop.getId()));
        return shop;
    }


    /**
     * 店铺详情
     * @param shopId
     * @param withOrders 否携带当日订单
     * @return
     */
    @Override
    public ShopDetailVO getShopDetail(Long shopId, Boolean withOrders) throws BusinessCheckException{
        // 1. 查询基础信息
        MtStore shop = shopMapper.selectById(shopId);
        if (shop == null) {
            throw new BusinessCheckException(I18nUtil.getMessage("storeNonexistent"));
        }

        // 2. 构建VO对象
        ShopDetailVO vo = new ShopDetailVO()
                .setId(shop.getId())
                .setName(shop.getName())
                .setMail(shop.getMail())
                .setNextReminderDay(shop.getNextReminderDay())
                .setAlertDesc(AlertEnum.getAlertDesc(shop.getAlertId()))
                .setThreshold(shop.getThreshold())
                .setCreateTime(shop.getCreateTime())
                .setUpdateTime(shop.getUpdateTime());

        // 3. 查询当日订单（按需加载）
        if (Boolean.TRUE.equals(withOrders)) {
            List<MtOrder> orders = orderMapper.selectList(
                    new LambdaQueryWrapper<MtOrder>()
                            .eq(MtOrder::getStoreId, shopId)
                            .ge(MtOrder::getCreateTime, LocalDate.now().atStartOfDay())
            );
            vo.setOrdersToday(convertToOrderVO(orders));
        }
        return vo;
    }

    /**
     * 分页查询店铺列表
     * @param page
     * @param size
     * @param name
     * @return
     */
    @Override
    public PageResult<ShopVO> listShops(Integer page, Integer size, String name,String nextReminderDay ) {
        // 1. 构建分页参数
        Page<MtStore> pageParam = new Page<>(page, size);
        Set<Integer> alertIds = AlertEnum.getAlertIds();
        // 2. 构建查询条件
        LambdaQueryWrapper<MtStore> wrapper = new LambdaQueryWrapper<>();
        wrapper.likeRight(StringUtils.hasText(name), MtStore::getName, name)
                .eq(StringUtils.hasText(nextReminderDay),MtStore::getNextReminderDay,nextReminderDay)
                .in(MtStore::getAlertId,alertIds)
                .select(MtStore::getId, MtStore::getName, MtStore::getMail,
                        MtStore::getContact, MtStore::getPhone,
                        MtStore::getAlertId,MtStore::getNextReminderDay,
                        MtStore::getThreshold, MtStore::getUpdateTime);

        // 3. 执行分页查询
        Page<MtStore> shopPage = (Page<MtStore>) shopMapper.selectPage(pageParam, wrapper);

        //计算每个店铺的总收益额
        // 4. 转换VO对象
        List<ShopVO> vos = shopPage.getRecords().stream()
                .map(shop -> {
                    ShopVO shopVO = new ShopVO();
                    BeanUtils.copyProperties(shop,shopVO);
                    shopVO.setAlertDesc(AlertEnum.getAlertDesc(shopVO.getAlertId()));
                    return shopVO;
                }).collect(Collectors.toList());

        // 5. 封装分页结果
        return new PageResult<>(
                shopPage.getCurrent(),
                shopPage.getSize(),
                shopPage.getTotal(),
                vos
        );
    }

    @Override
    public List<AlertDTO> alertInterval() {
	    return AlertEnum.getAlertEnumList().stream()
                .map(AlertDTO::new)
                .collect(Collectors.toList());
    }

    @Override
    public List<SelectDto> earlyWarningStore(String storeName) {
        Set<Integer> alertIds = AlertEnum.getAlertIds();
        List<MtStore> mtStores = shopMapper.selectList(new LambdaQueryWrapper<MtStore>()
                .likeRight(StringUtils.hasText(storeName), MtStore::getName, storeName)
                .notIn(MtStore::getAlertId, alertIds));
	    return mtStores.stream().map(store -> {
            SelectDto selectDto = new SelectDto();
            selectDto.setValue(store.getId());
            selectDto.setLabel(store.getName());
            return selectDto;
        }).collect(Collectors.toList());
    }


    /**
     * 参数转换  MtOrder-》vo
     * @param orders
     * @return
     */
    private List<OrderVO> convertToOrderVO(List<MtOrder> orders) {
        return orders.stream()
                .map(o -> new OrderVO(o.getId(), o.getAmount(), o.getCreateTime()))
                .collect(Collectors.toList());
    }
}
