package com.fuint.module.backendApi.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fuint.common.dto.AccountInfo;
import com.fuint.common.dto.GoodsTopDto;
import com.fuint.common.dto.MemberTopDto;
import com.fuint.common.dto.Page;
import com.fuint.common.dto.ext.TransactionsDto;
import com.fuint.common.service.GoodsService;
import com.fuint.common.service.MemberService;
import com.fuint.common.service.OrderService;
import com.fuint.common.service.RefundService;
import com.fuint.common.util.DateUtil;
import com.fuint.common.util.I18nUtil;
import com.fuint.common.util.TokenUtil;
import com.fuint.framework.exception.BusinessCheckException;
import com.fuint.framework.web.BaseController;
import com.fuint.framework.web.ResponseObject;
import com.fuint.repository.model.MtOrder;
import com.fuint.repository.model.MtRefund;
import com.fuint.utils.StringUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.text.ParseException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 数据统计控制器
 *
 * Created by FSQ
 * CopyRight https://www.fuint.cn
 */
@Api(tags="管理端-数据统计相关接口")
@RestController
@AllArgsConstructor
@RequestMapping(value = "/backendApi/statistic")
public class BackendStatisticController extends BaseController {

    /**
     * 会员服务接口
     * */
    private MemberService memberService;

    /**
     * 订单服务接口
     * */
    private OrderService orderService;

    /**
     * 商品服务接口
     * */
    private GoodsService goodsService;


    private RefundService refundService;

    /**
     * 数据概况
     *
     * @return
     */
    @ApiOperation(value = "数据概况")
    @RequestMapping(value = "/main", method = RequestMethod.POST)
    
    public ResponseObject main(HttpServletRequest request, @RequestBody Map<String, Object> param) throws BusinessCheckException, ParseException {
        String token = request.getHeader("Access-Token");
        String startTimeStr = param.get("startTime") == null ? "" : param.get("startTime").toString();
        String endTimeStr = param.get("endTime") == null ? "" : param.get("endTime").toString();

        Date startTime = StringUtil.isNotEmpty(startTimeStr) ? DateUtil.parseDate(startTimeStr) : null;
        Date endTime = StringUtil.isNotEmpty(endTimeStr) ? DateUtil.parseDate(endTimeStr) : null;

        AccountInfo accountInfo = TokenUtil.getAccountInfoByToken(token);
        if (accountInfo == null) {
            return getFailureResult(1001, I18nUtil.getMessage("notAuthenticated"));
        }

        Long merchantId = accountInfo.getMerchantId();
        Long storeId = accountInfo.getStoreId();

        // 总会员数
        Long totalUserCount = memberService.getUserCount(merchantId, storeId);
        // 新增会员数量
        Long userCount = memberService.getUserCount(merchantId, storeId, startTime, endTime);

        // 总订单数
        BigDecimal totalOrderCount = orderService.getOrderCount(merchantId, storeId);
        // 订单数
        BigDecimal orderCount = orderService.getOrderCount(merchantId, storeId, startTime, endTime);

        // 交易金额
        BigDecimal payAmount = orderService.getPayMoney(merchantId, storeId, startTime, endTime);

        //获取支付后退款的订单
        List<MtOrder> refundOrder = orderService.refundOrder(merchantId, storeId, startTime, endTime);
        Set<Long> refundOrderIds = refundOrder.stream().map(MtOrder::getId).collect(Collectors.toSet());
        BigDecimal totalRefundAmount;
        if (!refundOrderIds.isEmpty()){
            totalRefundAmount = refundService.list(new LambdaQueryWrapper<MtRefund>()
                            .in(MtRefund::getOrderId, refundOrderIds)).stream()
                    .map(MtRefund::getAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
            payAmount = payAmount.subtract(totalRefundAmount);
        }else{
            totalRefundAmount = new BigDecimal("0");
        }

        // 总交易金额
        BigDecimal totalPayAmount = orderService.getPayMoney(merchantId, storeId, startTime, endTime);

        // 活跃会员数
//        Long activeUserCount = memberService.getActiveUserCount(merchantId, storeId, startTime, endTime);

        // 总支付人数
        Integer totalPayUserCount = orderService.getPayUserCount(merchantId, storeId);


        Map<String, Object> result = new HashMap<>();

        result.put("userCount", userCount);
        result.put("totalUserCount", totalUserCount);
        result.put("orderCount", orderCount);
        result.put("totalOrderCount", totalOrderCount);
        result.put("payAmount", payAmount);
        result.put("totalPayAmount", totalPayAmount);
        result.put("totalRefundAmount", totalRefundAmount);
//        result.put("activeUserCount", activeUserCount);
        result.put("totalPayUserCount", totalPayUserCount);

        return getSuccessResult(result);
    }

    /**
     * 获取交易列表
     *
     * @param page      分页参数
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return 结果
     */
    @ApiOperation(value = "获取交易列表")
    @RequestMapping(value = "/getTransactionsList", method = RequestMethod.GET)
    
    public ResponseObject getTransactionsList(Page page,
                                              @RequestParam("startTime") String startTime,
                                              @RequestParam("endTime") String endTime){
        List<TransactionsDto> transactionsBeanList = orderService.getTransactionsList(page,startTime,endTime);
        return getSuccessResult(transactionsBeanList);
    }

    /**
     * 排行榜数据
     *
     * @return
     */
    @ApiOperation(value = "排行榜数据")
    @RequestMapping(value = "/top", method = RequestMethod.POST)
    
    public ResponseObject top(HttpServletRequest request, @RequestBody Map<String, Object> param) throws ParseException {
        String token = request.getHeader("Access-Token");
        String startTimeStr = param.get("startTime") == null ? "" : param.get("startTime").toString();
        String endTimeStr = param.get("endTime") == null ? "" : param.get("endTime").toString();

        Date startTime = StringUtil.isNotEmpty(startTimeStr) ? DateUtil.parseDate(startTimeStr) : null;
        Date endTime = StringUtil.isNotEmpty(endTimeStr) ? DateUtil.parseDate(endTimeStr) : null;

        AccountInfo accountInfo = TokenUtil.getAccountInfoByToken(token);
        if (accountInfo == null) {
            return getFailureResult(1001, I18nUtil.getMessage("notAuthenticated"));
        }

        Long merchantId = accountInfo.getMerchantId();
        Long storeId = accountInfo.getStoreId();

        Map<String, Object> result = new HashMap<>();

        List<GoodsTopDto> goodsList = goodsService.getGoodsSaleTopList(merchantId, storeId, startTime, endTime);
        List<MemberTopDto> memberList = memberService.getMemberConsumeTopList(merchantId, storeId, startTime, endTime);

        result.put("goodsList", goodsList);
        result.put("memberList", memberList);

        return getSuccessResult(result);
    }

    /**
     * 获取会员数量
     *
     * @return
     */
    @ApiOperation(value = "获取会员数量")
    @RequestMapping(value = "/totalMember", method = RequestMethod.GET)
    
    public ResponseObject totalMember(HttpServletRequest request) throws BusinessCheckException {
        String token = request.getHeader("Access-Token");
        AccountInfo accountInfo = TokenUtil.getAccountInfoByToken(token);
        if (accountInfo == null) {
            return getFailureResult(1001, I18nUtil.getMessage("notAuthenticated"));
        }

        Long merchantId = accountInfo.getMerchantId();
        Long storeId = accountInfo.getStoreId();

        Long totalMember = memberService.getUserCount(merchantId, storeId);
        Map<String, Object> result = new HashMap<>();
        result.put("totalMember", totalMember);

        return getSuccessResult(result);
    }
}
