package com.fuint.websocket;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.websocket.OnClose;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArraySet;

@Component
@Slf4j
@ServerEndpoint("/websocket/{storeId}")
public class WebSocket {

	/**
	 * 线程安全的无序的集合
	 */
	private static final CopyOnWriteArraySet<Session> SESSIONS = new CopyOnWriteArraySet<>();

	/**
	 * 存储在线连接数
	 */
	private static final Map<String, Session> SESSION_POOL = new HashMap<>();

	@OnOpen
	public void onOpen(Session session, @PathParam(value = "storeId") String storeId) {
		try {
			SESSIONS.add(session);
			SESSION_POOL.put(storeId, session);
			log.info("【WebSocket消息】有新的连接，总数为：" + SESSIONS.size());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@OnClose
	public void onClose(Session session) {
		try {
			// 从session属性获取storeId
			String storeId = (String) session.getUserProperties().get("storeId");
			log.info("storeId--------->{}",storeId);
			// 安全移除会话
			if (storeId != null) {
				// 只移除当前对应的session（避免新会话被旧关闭操作影响）
				SESSION_POOL.remove(storeId, session); // 使用remove(key, value)保证原子性
			}

			SESSIONS.remove(session);
			log.info("【WebSocket消息】连接断开，总数为：" + SESSIONS.size());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@OnMessage
	public void onMessage(String message) {
		log.info("【WebSocket消息】收到客户端消息：" + message);
	}

	/**
	 * 此为广播消息
	 *
	 * @param message 消息
	 */
	public void sendAllMessage(String message) {
		log.info("【WebSocket消息】广播消息：" + message);
		for (Session session : SESSIONS) {
			try {
				if (session.isOpen()) {
					session.getAsyncRemote().sendText(message);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * 此为单点消息
	 *
	 * @param storeId  用户编号
	 * @param message 消息
	 */
	public void sendOneMessage(String storeId, String message) {
		Session session = SESSION_POOL.get(storeId);
		if (session != null && session.isOpen()) {
			try {
				synchronized (session) {
					log.info("【WebSocket消息】单点消息：" + message);
					session.getAsyncRemote().sendText(message);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * 此为单点消息(多人)
	 *
	 * @param storeIds 用户编号列表
	 * @param message 消息
	 */
	public void sendMoreMessage(String[] storeIds, String message) {
		for (String storeId : storeIds) {
			Session session = SESSION_POOL.get(storeId);
			if (session != null && session.isOpen()) {
				try {
					log.info("【WebSocket消息】单点消息：" + message);
					session.getAsyncRemote().sendText(message);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
	}
}
